/*
  Runs a single switch controller. Controller is selected by the
  CONTROLLER_ID #define. In a system with several switches, this enables all
  of them to run same source file.

  Each controller has its own hostname and 1-4 device names. The ports are
  used in the following order: GPIO2, GPIO0, GPIO3 (RX), GPIO1 (TX).

*/

#include <ESP8266WiFi.h>
#include <ArduinoOTA.h>
#include <WiFiClient.h>
#include <Thing.h>
#include <WebThingAdapter.h>

// valid values are 1-5
#define CONTROLLER_ID 3

#ifndef STASSID
#define STASSID "your_network_ssid"
#define STAPSK  "your_network_password"
#endif

const char* ssid = STASSID;
const char* sspassword = STAPSK;

const char *esp_passwd = "OTA_update_password" ;

#define GPIO0 0
#define TX 1
#define GPIO2 2
#define RX 3
#define DELAY 3000

struct onoff {
  int id ;
  const char *name ;
  int on ;
  int off ;
  ThingDevice *dev ;
}  ;

#if CONTROLLER_ID == 1
#define N_SWITCHES 4
  const char* myname = "switch1" ;
  const char* switch_names[4] =
  			{ "device01", "device02", "device03", "device04" } ;
#elif CONTROLLER_ID == 2
#define N_SWITCHES 3
  const char* myname = "switch2" ;
  const char* switch_names[4] = 
  			{ "device05", "device06", "device07", "unused" } ;
#elif CONTROLLER_ID == 3
#define N_SWITCHES 2
  const char* myname = "switch3" ;
  const char* switch_names[4] = { "device08", "device09", "unused", "unused" } ;
#elif CONTROLLER_ID == 4
#define N_SWITCHES 1
  const char* myname = "switch4" ;
  const char* switch_names[4] = { "zone10", "unused", "unused", "unused" } ;
#endif


struct onoff pins[4] = {
  { GPIO2, nullptr, LOW, HIGH, nullptr },
  { GPIO0, nullptr, LOW, HIGH, nullptr },
  { RX, nullptr, LOW, HIGH, nullptr },
  { TX, nullptr, LOW, HIGH, nullptr }
} ;

const char *switchTypes[] = {"OnOffSwitch", "Light", nullptr};
WebThingAdapter *adapter ;

ThingProperty *onoff[4] ;

void setup() {
  int i ;
  
  for ( i=0 ; i < N_SWITCHES ; i++ ) {
    pinMode(pins[i].id, OUTPUT);
    digitalWrite(pins[i].id, pins[i].off );
    pins[i].name = switch_names[i] ;
    pins[i].dev = new ThingDevice(  switch_names[i], switch_names[i], switchTypes ) ;
    onoff[i] = new ThingProperty("on",
      "Whether the switch is turned on",
      BOOLEAN, "OnOffProperty") ;
  }
  WiFi.hostname( myname ) ;
  WiFi.mode( WIFI_STA ) ;
  WiFi.begin( ssid, sspassword ) ;
  
  while ( WiFi.status() != WL_CONNECTED ) {
    delay( 500 ) ;
  }

  ArduinoOTA.setHostname( myname ) ;
  ArduinoOTA.setPassword( esp_passwd ) ;

  adapter = new WebThingAdapter( myname, WiFi.localIP());

  for ( i = 0 ; i < N_SWITCHES ; i++ ) {
  	pins[i].dev->addProperty( onoff[i] ) ;
  	adapter->addDevice( pins[i].dev ) ;
  }
  adapter->begin();
}

// the loop function runs over and over again forever
void loop() {
  bool am_on ;
  int i ;
  ArduinoOTA.handle();
  adapter->update();
  for ( i = 0 ; i < N_SWITCHES ; i++ ) {
    am_on = onoff[i]->getValue().boolean;
    digitalWrite(pins[i].id, am_on ? pins[i].on : pins[i].off );
  }
}
